@@ -142,8 +142,6 @@ class GaussianMLP(Ensemble):
 
         self.elite_models: List[int] = None
 
-        self._propagation_indices: torch.Tensor = None
-
     def _maybe_toggle_layers_use_only_elite(self, only_elite: bool):
         if self.elite_models is None:
             return
@@ -198,6 +196,7 @@ class GaussianMLP(Ensemble):
         self,
         x: torch.Tensor,
         rng: Optional[torch.Generator] = None,
+        propagation_indices: Optional[torch.Tensor] = None,
     ) -> Tuple[torch.Tensor, Optional[torch.Tensor]]:
         if self.propagation_method is None:
             mean, logvar = self._default_forward(x, only_elite=False)
@@ -222,7 +221,11 @@ class GaussianMLP(Ensemble):
             model_indices = torch.randperm(x.shape[1], device=self.device)
             return self._forward_from_indices(x, model_indices)
         if self.propagation_method == "fixed_model":
-            return self._forward_from_indices(x, self._propagation_indices)
+            if propagation_indices is None:
+                raise ValueError(
+                    "When using propagation='fixed_model', `propagation_indices` must be provided."
+                )
+            return self._forward_from_indices(x, propagation_indices)
         if self.propagation_method == "expectation":
             mean, logvar = self._default_forward(x, only_elite=True)
             return mean.mean(dim=0), logvar.mean(dim=0)
@@ -232,6 +235,7 @@ class GaussianMLP(Ensemble):
         self,
         x: torch.Tensor,
         rng: Optional[torch.Generator] = None,
+        propagation_indices: Optional[torch.Tensor] = None,
         use_propagation: bool = True,
     ) -> Tuple[torch.Tensor, torch.Tensor]:
         """Computes mean and logvar predictions for the given input.
@@ -264,6 +268,9 @@ class GaussianMLP(Ensemble):
                 the shape must be ``B x Id``.
             rng (torch.Generator, optional): random number generator to use for "random_model"
                 propagation.
+            propagation_indices (tensor, optional): propagation indices to use,
+                as generated by :meth:`sample_propagation_indices`. Ignore if
+                `use_propagation == False` or `self.propagation_method != "fixed_model".
             use_propagation (bool): if ``False``, the propagation method will be ignored
                 and the method will return outputs for all models. Defaults to ``True``.
 
@@ -284,7 +291,9 @@ class GaussianMLP(Ensemble):
 
         """
         if use_propagation:
-            return self._forward_ensemble(x, rng=rng)
+            return self._forward_ensemble(
+                x, rng=rng, propagation_indices=propagation_indices
+            )
         return self._default_forward(x)
 
     def _mse_loss(self, model_in: torch.Tensor, target: torch.Tensor) -> torch.Tensor:
@@ -367,30 +376,9 @@ class GaussianMLP(Ensemble):
             target = target.repeat((self.num_members, 1, 1))
             return F.mse_loss(pred_mean, target, reduction="none"), {}
 
-    def reset(  # type: ignore
-        self, x: torch.Tensor, rng: Optional[torch.Generator] = None
-    ) -> torch.Tensor:
-        """Initializes any internal dependent state when using the model for simulation.
-
-        Initializes model indices for "fixed_model" propagation method
-        a bootstrapped ensemble with TSinf propagation).
-
-        Args:
-            x (tensor): the input to the model.
-            rng (random number generator): a rng to use for sampling the model
-                indices.
-
-        Returns:
-            (tensor): forwards the same input.
-        """
-        assert rng is not None
-        self._propagation_indices = self._sample_propagation_indices(x.shape[0], rng)
-        return x
-
-    def _sample_propagation_indices(
+    def sample_propagation_indices(
         self, batch_size: int, _rng: torch.Generator
     ) -> torch.Tensor:
-        """Returns a random permutation of integers in [0, ``batch_size``)."""
         model_len = (
             len(self.elite_models) if self.elite_models is not None else len(self)
         )
