@@ -5,7 +5,7 @@
 import pathlib
 import pickle
 import warnings
-from typing import List, Optional, Sequence, Tuple, Union
+from typing import Dict, List, Optional, Sequence, Tuple, Union
 
 import numpy as np
 import torch
@@ -42,6 +42,9 @@ class OneDTransitionRewardModel(Model):
 
         [pred_obs_{t+1}, pred_rewards_{t+1} (optional)] = model([obs_t, action_t]).
 
+    To use with :class:mbrl.models.ModelEnv`, the wrapped model must define methods
+    ``reset_1d`` and ``sample_1d``.
+
     Args:
         model (:class:`mbrl.model.Model`): the model to wrap.
         target_is_delta (bool): if ``True``, the predicted observations will represent
@@ -50,7 +53,8 @@ class OneDTransitionRewardModel(Model):
             Defaults to ``True``. Can be deactivated per dimension using ``no_delta_list``.
         normalize (bool): if true, the wrapper will create a normalizer for model inputs,
             which will be used every time the model is called using the methods in this
-            class. To update the normalizer statistics, the user needs to call
+            class. Assumes the given base model has an attributed ``in_size``.
+            To update the normalizer statistics, the user needs to call
             :meth:`update_normalizer` before using the model. Defaults to ``False``.
         normalize_double_precision (bool): if ``True``, the normalizer will work with
             double precision.
@@ -82,7 +86,7 @@ class OneDTransitionRewardModel(Model):
         no_delta_list: Optional[List[int]] = None,
         num_elites: Optional[int] = None,
     ):
-        super().__init__()
+        super().__init__(model.device)
         self.model = model
         self.input_normalizer: Optional[mbrl.util.math.Normalizer] = None
         if normalize:
@@ -106,26 +110,21 @@ class OneDTransitionRewardModel(Model):
             else None
         )
 
-    def _get_model_input_from_np(
-        self, obs: np.ndarray, action: np.ndarray, device: torch.device
+    def _get_model_input(
+        self,
+        obs: mbrl.types.TensorType,
+        action: mbrl.types.TensorType,
     ) -> torch.Tensor:
         if self.obs_process_fn:
             obs = self.obs_process_fn(obs)
-        model_in_np = np.concatenate([obs, action], axis=obs.ndim - 1)
-        if self.input_normalizer:
-            # Normalizer lives on device
-            return self.input_normalizer.normalize(model_in_np).float().to(device)
-        return torch.from_numpy(model_in_np).to(device)
-
-    def _get_model_input_from_tensors(self, obs: torch.Tensor, action: torch.Tensor):
-        if self.obs_process_fn:
-            obs = self.obs_process_fn(obs)
-        model_in = torch.cat([obs, action], axis=obs.ndim - 1)
+        obs = model_util.to_tensor(obs).to(self.device)
+        action = model_util.to_tensor(action).to(self.device)
+        model_in = torch.cat([obs, action], dim=obs.ndim - 1)
         if self.input_normalizer:
             model_in = self.input_normalizer.normalize(model_in).float()
         return model_in
 
-    def _get_model_input_and_target_from_batch(
+    def _process_batch(
         self, batch: mbrl.types.TransitionBatch
     ) -> Tuple[torch.Tensor, torch.Tensor]:
         obs, action, next_obs, reward, _ = batch.astuple()
@@ -135,22 +134,15 @@ class OneDTransitionRewardModel(Model):
                 target_obs[..., dim] = next_obs[..., dim]
         else:
             target_obs = next_obs
+        target_obs = model_util.to_tensor(target_obs).to(self.device)
 
-        model_in = self._get_model_input_from_np(obs, action, self.device)
+        model_in = self._get_model_input(obs, action)
         if self.learned_rewards:
-            target = (
-                torch.from_numpy(
-                    np.concatenate(
-                        [target_obs, np.expand_dims(reward, axis=reward.ndim)],
-                        axis=obs.ndim - 1,
-                    )
-                )
-                .float()
-                .to(self.device)
-            )
+            reward = model_util.to_tensor(reward).to(self.device).unsqueeze(reward.ndim)
+            target = torch.cat([target_obs, reward], dim=obs.ndim - 1)
         else:
-            target = torch.from_numpy(target_obs).float().to(self.device)
-        return model_in, target
+            target = target_obs
+        return model_in.float(), target.float()
 
     def forward(self, x: torch.Tensor, **kwargs) -> Tuple[torch.Tensor, ...]:
         """Calls forward method of base model with the given input and args."""
@@ -196,7 +188,7 @@ class OneDTransitionRewardModel(Model):
             (tensor and optional dict): as returned by `model.loss().`
         """
         assert target is None
-        model_in, target = self._get_model_input_and_target_from_batch(batch)
+        model_in, target = self._process_batch(batch)
         return self.model.loss(model_in, target=target)
 
     def update(
@@ -215,7 +207,7 @@ class OneDTransitionRewardModel(Model):
             (tensor and optional dict): as returned by `model.loss().`
         """
         assert target is None
-        model_in, target = self._get_model_input_and_target_from_batch(batch)
+        model_in, target = self._process_batch(batch)
         return self.model.update(model_in, optimizer, target=target)
 
     def eval_score(
@@ -236,7 +228,7 @@ class OneDTransitionRewardModel(Model):
         """
         assert target is None
         with torch.no_grad():
-            model_in, target = self._get_model_input_and_target_from_batch(batch)
+            model_in, target = self._process_batch(batch)
             return self.model.eval_score(model_in, target=target)
 
     def get_output_and_targets(
@@ -255,24 +247,30 @@ class OneDTransitionRewardModel(Model):
             (tuple(tensor), tensor): the model outputs and the target for this batch.
         """
         with torch.no_grad():
-            model_in, target = self._get_model_input_and_target_from_batch(batch)
+            model_in, target = self._process_batch(batch)
             output = self.model.forward(model_in)
         return output, target
 
-    def sample(  # type: ignore
+    def sample(
         self,
-        x: mbrl.types.TransitionBatch,
+        act: torch.Tensor,
+        model_state: Dict[str, torch.Tensor],
         deterministic: bool = False,
         rng: Optional[torch.Generator] = None,
-        **kwargs,
-    ) -> Tuple[torch.Tensor, torch.Tensor]:
-        """Samples next observations and rewards from the underlying model.
+    ) -> Tuple[
+        torch.Tensor,
+        Optional[torch.Tensor],
+        Optional[torch.Tensor],
+        Optional[Dict[str, torch.Tensor]],
+    ]:
+        """Samples next observations and rewards from the underlying 1-D model.
 
         This wrapper assumes that the underlying model's sample method returns a tuple
         with just one tensor, which concatenates next_observation and reward.
 
         Args:
-            x (transition): a batch of transitions.
+            act (tensor): the action at.
+            model_state (tensor): the model state st.
             deterministic (bool): if ``True``, the model returns a deterministic
                 "sample" (e.g., the mean prediction). Defaults to ``False``.
             rng (random number generator): a rng to use for sampling.
@@ -280,11 +278,15 @@ class OneDTransitionRewardModel(Model):
         Returns:
             (tuple of two tensors): predicted next_observation (o_{t+1}) and rewards (r_{t+1}).
         """
-        obs = model_util.to_tensor(x.obs).to(self.device)
-        actions = model_util.to_tensor(x.act).to(self.device)
-
-        model_in = self._get_model_input_from_tensors(obs, actions)
-        preds = self.model.sample(model_in, rng=rng, deterministic=deterministic)[0]
+        obs = model_util.to_tensor(model_state["obs"]).to(self.device)
+        model_in = self._get_model_input(model_state["obs"], act)
+        if not hasattr(self.model, "sample_1d"):
+            raise RuntimeError(
+                "OneDTransitionRewardModel requires wrapped model to define method sample_1d"
+            )
+        preds, next_model_state = self.model.sample_1d(
+            model_in, model_state, rng=rng, deterministic=deterministic
+        )
         next_observs = preds[:, :-1] if self.learned_rewards else preds
         if self.target_is_delta:
             tmp_ = next_observs + obs
@@ -292,23 +294,31 @@ class OneDTransitionRewardModel(Model):
                 tmp_[:, dim] = next_observs[:, dim]
             next_observs = tmp_
         rewards = preds[:, -1:] if self.learned_rewards else None
-        return next_observs, rewards
+        next_model_state["obs"] = next_observs
+        return next_observs, rewards, None, next_model_state
 
-    def reset(  # type: ignore
-        self, x: mbrl.types.TransitionBatch, rng: Optional[torch.Generator] = None
-    ) -> torch.Tensor:
+    def reset(
+        self, obs: torch.Tensor, rng: Optional[torch.Generator] = None
+    ) -> Dict[str, torch.Tensor]:
         """Calls reset on the underlying model.
 
         Args:
-            x (tensor): the input to the model.
-            rng (random number generator): a rng to use for sampling the model
-                indices.
+            obs (tensor): the observation from which the trajectory will be
+                started. The actual value is ignore, only the shape is used.
+            rng (`torch.Generator`, optional): an optional random number generator
+                to use.
 
         Returns:
-            (tensor): the output of the underlying model.
+            (dict(str, tensor)): the model state necessary to continue the simulation.
         """
-        obs = model_util.to_tensor(x.obs).to(self.device)
-        return self.model.reset(obs, rng=rng)
+        if not hasattr(self.model, "reset_1d"):
+            raise RuntimeError(
+                "OneDTransitionRewardModel requires wrapped model to define method reset_1d"
+            )
+        obs = model_util.to_tensor(obs).to(self.device)
+        model_state = {"obs": obs}
+        model_state.update(self.model.reset_1d(obs, rng=rng))
+        return model_state
 
     # TODO replace this with calls to self.model.save() and self.model.load() in next version
     def save(self, save_dir: Union[str, pathlib.Path]):
