@@ -5,18 +5,9 @@
 from .core import Agent, RandomAgent, complete_agent_cfg, load_agent
 from .trajectory_opt import (
     CEMOptimizer,
+    ICEMOptimizer,
+    MPPIOptimizer,
     TrajectoryOptimizer,
     TrajectoryOptimizerAgent,
     create_trajectory_optim_agent_for_model,
 )
-
-__all__ = [
-    "Agent",
-    "CEMOptimizer",
-    "RandomAgent",
-    "TrajectoryOptimizer",
-    "TrajectoryOptimizerAgent",
-    "complete_agent_cfg",
-    "create_trajectory_optim_agent_for_model",
-    "load_agent",
-]
