@@ -3,6 +3,9 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 from .logger import Logger
-from .replay_buffer import ReplayBuffer, SequenceTransitionIterator, TransitionIterator
-
-__all__ = ["Logger", "ReplayBuffer", "TransitionIterator", "SequenceTransitionIterator"]
+from .replay_buffer import (
+    ReplayBuffer,
+    SequenceTransitionIterator,
+    SequenceTransitionSampler,
+    TransitionIterator,
+)
