@@ -14,7 +14,7 @@ import mbrl.util.common as utils
 
 class MockModel(models.Model):
     def __init__(self, x, y, in_size, out_size):
-        super().__init__()
+        super().__init__(None)
         self.in_size = in_size
         self.out_size = out_size
         self.x = x
@@ -136,9 +136,9 @@ class MockModelEnv:
 
     def reset(self, obs0, return_as_np=None):
         self.obs = obs0
-        return obs0
+        return {}
 
-    def step(self, action, sample=None):
+    def step(self, action, model_state, sample=None):
         next_obs = self.obs + action[:, :1]
         reward = np.ones(next_obs.shape[0])
         done = np.zeros(next_obs.shape[0])
